<?php

namespace App\Http\Controllers\Cms;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Post;
use App\Models\Category;
use App\Models\Tag;

use Image;
use Session;
use Auth;

class PostsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $posts = Post::orderBy('created_at', 'desc')->get();
        return view('backend.admin.posts.index')->with('posts', $posts);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $categories = Category::orderBy('created_at', 'desc')->get();
        $tags = Tag::orderBy('created_at', 'desc')->get();

        if($categories->count() == 0){
            Session::flash('warning', 'Category not found ! Insert a new category to create a post.');
            return redirect()->route('add.category');
        } else if($tags->count() == 0){
            Session::flash('warning', 'Tags not found ! Insert a new tag to create a post.');
            return redirect()->route('add.tag');
        }

        return view('backend.admin.posts.create', compact('categories', 'tags'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'title'       => 'required|unique:posts',
            'image'       => 'required',
            'author'      => 'required',
            'category_id' => 'required',
            'tags'        => 'required',
            'paragraph'   => 'required|array',
        ]);


        $post = new Post();

        $post->title = $request->title;
        $post->slug = str_slug($request->title);
        $post->author = $request->author;
        $post->category_id = $request->category_id;
        $post->quote = $request->quote;

        if ($request->hasFile('image')) {
            $image_tmp = $request->file('image');
            if ($image_tmp->isValid()) {
                $image_name = $image_tmp->getClientOriginalName();
                $extension = $image_tmp->getClientOriginalExtension();
                $image_new_name = $image_name.'-'.rand(111,99999).'.'.$extension;

                $original_image_path = 'assets/uploads/posts/'.$image_new_name;
                
                Image::make($image_tmp)->save($original_image_path);
                
                $post->image = $image_new_name;
            }
        }

        $post->paragraph = json_encode($request->paragraph);

        $post->save();

        $post->tags()->attach($request->tags);

        Session::flash('success', 'Blog Post Created Successfully !');
        return redirect()->back();
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {

        $post = Post::findOrFail($id);
        $tags = Tag::all();
        $categories = Category::orderBy('created_at', 'desc')->get();

        $tag_ids = [];
        foreach($post->tags as $potag) {
            array_push($tag_ids, $potag->id);
        }

        $paragraphs = json_decode($post->paragraph);
        
        return view('backend.admin.posts.edit', compact('post', 'categories', 'tags', 'tag_ids', 'paragraphs'));

    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'title'       => 'required|unique:posts,title,'.$id,
            'author'      => 'required',
            'category_id' => 'required',
            'tags'        => 'required',
            'paragraph'   => 'required|array',
        ]);


        $post = Post::findOrFail($id);

        $post->title = $request->title;
        $post->slug = str_slug($request->title);
        $post->author = $request->author;
        $post->category_id = $request->category_id;
        $post->quote = $request->quote;

        if ($request->hasFile('image')) {
            $image_tmp = $request->file('image');
            if ($image_tmp->isValid()) {
                $image_name = $image_tmp->getClientOriginalName();
                $extension = $image_tmp->getClientOriginalExtension();
                $image_new_name = $image_name.'-'.rand(111,99999).'.'.$extension;

                $original_image_path = 'assets/uploads/posts/'.$image_new_name;
                
                Image::make($image_tmp)->save($original_image_path);
                
                $post->image = $image_new_name;
            }
        }

        $post->paragraph = json_encode($request->paragraph);

        $post->save();

        $post->tags()->sync($request->tags);

        Session::flash('info', 'Post Updated Successfully !');

        return redirect()->route('all.posts');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $category = Post::findOrFail($id);

        $category->delete();

        Session::flash("error", "Post deleted !");

        return redirect()->route('all.posts'); 
    }
}
