<?php

namespace App\Http\Controllers\Cms;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Post;
use App\Models\Category;
use App\Models\Tag;
use App\Models\ProductCategory;
use App\Models\Product;
use App\Models\Order;

use Image;
use Session;
use Auth;

class ProductsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $products = Product::orderBy('created_at', 'desc')->get();
        return view('backend.admin.products.index')->with('products', $products);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $pro_categories = ProductCategory::orderBy('created_at', 'desc')->get();
        
        if($pro_categories->count() == 0){
            Session::flash('warning', 'Category not found ! Insert a new category to create a new product.');
            return redirect()->route('add.product.category');
        }

        return view('backend.admin.products.create', compact('pro_categories'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name'        => 'required|unique:products',
            'images'      => 'required',
            'category_id' => 'required',
            'price'       => 'required',
            'description' => 'required',
        ]);


        $product = new Product();

        $product->name = $request->name;
        $product->slug  = str_slug($request->name);
        $product->category_id = $request->category_id;
        $product->price = $request->price;
        $product->description = $request->description;
        $product->additional_info = $request->additional_info;
        $product->payment_info = $request->payment_info;

        $images_array = [];

        if($request->hasFile('images')) {
            
            $files = $request->file('images');

            foreach($files as $image_tmp) {
                
                if ($image_tmp->isValid()) {
                    $image_name = $image_tmp->getClientOriginalName();
                    $extension = $image_tmp->getClientOriginalExtension();
                    $image_new_name = $image_name.'-'.rand(11,9999999).'.'.$extension;

                    $original_image_path = 'assets/uploads/products/'.$image_new_name;
                    
                    Image::make($image_tmp)->save($original_image_path);
                    
                    array_push($images_array, $image_new_name);
                }

            }
        }

        $product->images = json_encode($images_array);

        $attributes_array = [];
        foreach($request['attributes'] as $attribute) {
            $attributes_array[$attribute['attribute_name']] = $attribute['attribute_value'];
        }

        $product->attributes = json_encode($attributes_array);
        
        $product->save();


        Session::flash('success', 'Product Added Successfully !');
        return redirect()->route('all.products');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {

        $product = Product::findOrFail($id);
        $pro_categories = ProductCategory::orderBy('created_at', 'desc')->get();

        $attributes = json_decode($product->attributes);
        
        return view('backend.admin.products.edit', compact('product', 'pro_categories', 'attributes'));

    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name'        => 'required|unique:products,name,'.$id,
            'category_id' => 'required',
            'price'        => 'required',
        ]);


        $product = Product::findOrFail($id);

        $product->name = $request->name;
        $product->slug  = str_slug($request->name);
        $product->category_id = $request->category_id;
        $product->price = $request->price;
        $product->description = $request->description;
        $product->additional_info = $request->additional_info;
        $product->payment_info = $request->payment_info;

        $previous_images = (array) json_decode($product->images, true);

        $images_array = [];

        if($request->hasFile('images')) {
            
            $files = $request->file('images');

            foreach($files as $image_tmp) {
                
                if ($image_tmp->isValid()) {
                    $image_name = $image_tmp->getClientOriginalName();
                    $extension = $image_tmp->getClientOriginalExtension();
                    $image_new_name = $image_name.'-'.rand(11,9999999).'.'.$extension;

                    $original_image_path = 'assets/uploads/products/'.$image_new_name;
                    
                    Image::make($image_tmp)->save($original_image_path);
                    
                    array_push($images_array, $image_new_name);
                }

            }

            $new_array = array_merge($images_array, $previous_images);

        } else {
            $new_array = $previous_images;
        }

        $product->images = json_encode($new_array);

        $attributes_array = [];
        foreach($request['attributes'] as $attribute) {
            $attributes_array[$attribute['attribute_name']] = $attribute['attribute_value'];
        }

        $product->attributes = json_encode($attributes_array);
        
        $product->save();


        Session::flash('info', 'Product Updated Successfully !');
        return redirect()->back();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $product = Product::findOrFail($id);

        $product->delete();

        Session::flash("error", "Product deleted !");

        return redirect()->route('all.products'); 
    }

    public function all_orders()
    {
        $orders = Order::orderBy('created_at', 'desc')->get();
        return view('backend.admin.products.all-orders')->with('orders', $orders);
    }

    public function invoice_details($id)
    {

        $order = Order::findOrFail($id);
        return view('backend.admin.products.order-details', compact('order'));

    }

}

